<?php
/**
 * WooCommerce Mollie
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Mollie to newer
 * versions in the future. If you wish to customize WooCommerce Mollie for your
 * needs please refer to http://docs.woocommerce.com/document/mollie-ideal/
 *
 * @author    SkyVerge
 * @copyright Copyright (c) 2011-2020, SkyVerge, Inc. (info@skyverge.com)
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

defined( 'ABSPATH' ) or exit;

use SkyVerge\WooCommerce\PluginFramework\v5_5_0 as Framework;

/**
 * WooCommerce Mollie Gateway main plugin class.
 *
 * @since 2.0
 *
 * @method \WC_Gateway_Mollie get_gateway( $gateway_id = null )
 */
class WC_Mollie extends Framework\SV_WC_Payment_Gateway_Plugin {


	/** version number */
	const VERSION = '2.15.2';

	/** @var WC_Mollie single instance of this plugin */
	protected static $instance;

	/** the plugin id */
	const PLUGIN_ID = 'mollie';

	/** the gateway class name */
	const GATEWAY_CLASS_NAME = 'WC_Gateway_Mollie';

	/** @var array associative array of child gateways slugs and names */
	protected $gateways;


	/**
	 * Sets up the main plugin class.
	 *
	 * @since 2.0
	 */
	public function __construct() {

		parent::__construct(
			self::PLUGIN_ID,
			self::VERSION,
			array(
				'text_domain'  => 'woocommerce-gateway-mollie',
				'gateways'     => array(
					self::PLUGIN_ID => self::GATEWAY_CLASS_NAME,
				),
				'dependencies' => array( 'curl' ),
				'currencies'   => array( 'EUR' ),
			)
		);

		$this->gateways = array(
			'ideal'        => 'WC_Gateway_Mollie_iDEAL',
			'creditcard'   => 'WC_Gateway_Mollie_CreditCard',
			'mistercash'   => 'WC_Gateway_Mollie_MisterCash',
			'banktransfer' => 'WC_Gateway_Mollie_BankTransfer',
			'bitcoin'      => 'WC_Gateway_Mollie_Bitcoin',
			'paypal'       => 'WC_Gateway_Mollie_PayPal',
			'paysafecard'  => 'WC_Gateway_Mollie_Paysafecard',
			'sofort'       => 'WC_Gateway_Mollie_SOFORT',
			'belfius'      => 'WC_Gateway_Mollie_Belfius',
		);

		add_filter( 'option_woocommerce_gateway_order', array( $this, 'fix_mollie_gateway_order' ) );
	}


	/**
	 * Initializes the plugin.
	 *
	 * @internal
	 *
	 * @since 2.13.0
	 */
	public function init_plugin() {

		$this->includes();
	}


	/**
	 * Loads any required files.
	 *
	 * @internal
	 *
	 * @since 2.0
	 */
	public function includes() {

		require_once( $this->get_plugin_path() . '/includes/wc-gateway-mollie-template.php' );

		$path = $this->get_plugin_path();

		// gateway classes
		require_once( $path . '/includes/class-wc-gateway-mollie.php' );

		foreach ( array_keys( $this->gateways ) as $gateway ) {

			if ( is_readable( $path . "/includes/class-wc-gateway-mollie-{$gateway}.php" ) ) {

				require_once( $path . "/includes/class-wc-gateway-mollie-{$gateway}.php" );
			}
		}
	}


	/**
	 * Initializes gateway template functions.
	 *
	 * TODO remove this method by version 3.0 or by December 2019, whichever comes first {FN 2019-01-02}
	 *
	 * @internal
	 *
	 * @since 2.0
	 * @deprecated since 2.13.0
	 */
	public function include_template_functions() {

		_deprecated_function( 'WC_Mollie::include_template_functions()', '2.13.0' );
	}


	/**
	 * Builds the lifecycle handler instance.
	 *
	 * @since 2.13.0
	 */
	protected function init_lifecycle_handler() {

		require_once( $this->get_plugin_path() . '/includes/Lifecycle.php' );

		$this->lifecycle_handler = new SkyVerge\WooCommerce\Mollie\Lifecycle( $this );
	}


	/**
	 * Adds any gateways supported by this plugin to the list of available payment gateways
	 *
	 * @internal
	 *
	 * @since 2.0
	 *
	 * @param string[] $gateways array
	 * @return string[]
	 */
	public function load_gateways( $gateways ) {

		// if we're in the admin or on the callback, use the normal behavior
		if ( $this->is_wc_api() || ( is_admin() && ! is_ajax() ) ) {
			return parent::load_gateways( $gateways );
		}

		// need to instantiate a gateway instance so we can pull the available methods
		$gateway = $this->get_gateway();

		try {

			foreach ( $gateway->get_methods() as $method ) {

				$gateway_class = isset( $this->gateways[ $method->id ] ) ? $this->gateways[ $method->id ] : null;

				if ( $gateway_class ) {
					$gateways[] = $gateway_class;
				}
			}

		} catch( \Mollie_API_Exception $e ) {

			if ( $gateway->debug_log() ) {

				$this->log( sprintf( 'Error getting methods: %s', $e->getMessage() ), $this->get_id() );
			}
		}

		/**
		 * Filters the Mollie gateways allowing plugins to reorder or remove gateways
		 *
		 * @since 2.4.4
		 *
		 * @param string[] $gateways an array of Mollie gateway classes to add as available gateways.
		 */
		return (array) apply_filters( 'wc_gateway_mollie_gateways', $gateways );
	}


	/**
	 * Returns true if the current request is for the woocommerce api endpoint.
	 *
	 * @since 2.0
	 *
	 * @return bool
	 */
	protected function is_wc_api() {

		return isset( $_REQUEST['wc-api'] ) && self::GATEWAY_CLASS_NAME === $_REQUEST['wc-api'];
	}


	/**
	 * Fix the Mollie child gateways ordering.
	 *
	 * Since Mollie is a special gateway which creates 5 "child" gateways each with their own id, we need to filter on the gateway ordering set by the admin to expand the single 'mollie' entry into the five possible options.
	 * This means you can't specify the ordering of each individually, but probably good enough to move them around as a block.
	 *
	 * @internal
	 *
	 * @since 2.1
	 *
	 * @param array $ordering associative array of gateway id to numerical order
	 * @return array associative array of gateway id to to numerical order
	 */
	public function fix_mollie_gateway_order( $ordering ) {

		if ( is_admin() && ! is_ajax() ) {
			return $ordering;
		}

		$new_ordering = array();
		$offset = 0;

		foreach ( $ordering as $gateway_id => $order ) {

			if ( self::PLUGIN_ID === $gateway_id ) {

				foreach ( array_keys( $this->gateways ) as $gateway ) {
					$new_ordering[ 'mollie_' . $gateway ] = $order++;
				}

				$offset = count( $this->gateways ) - 1;
			} else {
				$new_ordering[ $gateway_id ] = $order + $offset;
			}
		}

		return $new_ordering;
	}


	/**
	 * Displays any admin notices that might depend on the plugin settings.
	 *
	 * This is called after the gateway settings are saved.
	 *
	 * @internal
	 *
	 * @since 2.4.0
	 */
	public function add_delayed_admin_notices() {

		parent::add_delayed_admin_notices();

		// look for sign of a legacy-API install, and instruct the admin to update their mollie account settings
		$settings = $this->get_gateway_settings( $this->get_id() );

		// legacy settings?
		if ( isset( $settings['partner_id'] ) ) {

			$this->get_admin_notice_handler()->add_admin_notice(
				__( 'This plugin now uses the updated Mollie API, in order to accept payments you will need to log onto your Mollie account and create an API Key, then update the plugin settings with that API Key.', 'woocommerce-gateway-mollie' ),
				'update-account-settings',
				array( 'notice_class' => 'error' )
			);
		}

		if ( $this->is_plugin_settings() && $this->get_gateway()->get_api_key() ) {

			// test the API key
			try {

				$this->get_gateway()->get_methods();

			} catch ( \Mollie_API_Exception $e ) {

				// communication error
				$this->get_admin_notice_handler()->add_admin_notice( __( 'Error connecting to Mollie with the supplied API Key, please verify and update the plugin settings', 'woocommerce-gateway-mollie' ), 'invalid-api-key', array( 'notice_class' => 'error' ) );

				$gateway = $this->get_gateway();

				if ( $gateway->debug_log() ) {

					$this->log( sprintf( 'Error retrieving methods: %s', $e->getMessage() ), $this->get_id() );
				}
			}
		}
	}


	/**
	 * Saves errors or messages to WooCommerce Log (woocommerce/logs/plugin-id-xxx.txt).
	 *
	 * @since 2.1
	 *
	 * @param string $message error or message to save to log
	 * @param string $log_id optional log id to segment the files by, defaults to plugin id
	 */
	public function log( $message, $log_id = null ) {

		// The Mollie payment gateway is implemented by multiple individual payment methods, each with their own id, but not really truly independent methods.
		// Therefore, when logging, always use the plugin id, so we have a single consolidated logfile.
		parent::log( $message, $this->get_id() );
	}


	/**
	 * Returns the main Mollie instance.
	 *
	 * Ensures only one instance is/can be loaded.
	 *
	 * @since 2.5.0
	 *
	 * @return \WC_Mollie
	 */
	public static function instance() {

		if ( null === self::$instance ) {

			self::$instance = new self();
		}

		return self::$instance;
	}


	/**
	 * Gets the plugin support URL.
	 *
	 * @since 2.6.0
	 *
	 * @return string
	 */
	public function get_support_url() {

		return 'https://woocommerce.com/my-account/marketplace-ticket-form/';
	}


	/**
	 * Gets the plugin documentation url.
	 *
	 * @since 2.0
	 *
	 * @return string documentation URL
	 */
	public function get_documentation_url() {

		return 'http://docs.woocommerce.com/document/mollie-ideal/';
	}


	/**
	 * Returns the plugin sales page URL.
	 *
	 * @since 2.13.0
	 *
	 * @return string
	 */
	public function get_sales_page_url() {

		return 'https://woocommerce.com/products/woocommerce-mollie/';
	}


	/**
	 * Returns the plugin name, localized.
	 *
	 * @since 2.0
	 *
	 * @return string the plugin name
	 */
	public function get_plugin_name() {

		return __( 'WooCommerce Mollie', 'woocommerce-gateway-mollie' );
	}


	/**
	 * Returns __FILE__.
	 *
	 * @since 2.0
	 *
	 * @return string the full path and filename of the plugin file
	 */
	protected function get_file() {

		return __FILE__;
	}


	/**
	 * Gets the gateway settings name.
	 *
	 * Opens the parent method to public.
	 *
	 * @since 2.13.0
	 *
	 * @param string $gateway_id
	 * @return string
	 */
	public function get_gateway_settings_name( $gateway_id ) {

		return parent::get_gateway_settings_name( $gateway_id );
	}


	/**
	 * Transforms the API request/response data into a string suitable for logging.
	 *
	 * @since 2.3.2
	 *
	 * @param mixed $data request/response data
	 * @return string
	 */
	public function get_api_log_message( $data ) {

		$messages = array();

		if ( $data instanceof \Mollie_API_Object_Payment ) {

			$data = array(
				'id'              => $data->id,
				'mode'            => $data->mode,
				'createdDatetime' => $data->createdDatetime,
				'status'          => $data->status,
				'amount'          => $data->amount,
				'description'     => $data->description,
				'method'          => $data->method,
				'metadata'        => $data->metadata,
				'details'         => $data->details,
				'links'           => $data->links,
			);
		}

		if ( is_array( $data ) ) {
			$messages[] = isset( $data['uri'] ) && $data['uri'] ? 'Request' : 'Response';

			foreach ( (array) $data as $key => $value ) {
				$messages[] = sprintf( '%s: %s', $key, is_array( $value ) || ( is_object( $value ) && 'stdClass' == get_class( $value ) ) ? print_r( (array) $value, true ) : $value );
			}
		}

		return implode( "\n", $messages );
	}


}


/**
 * Returns the One True Instance of Mollie.
 *
 * @since 2.5.0
 *
 * @return WC_Mollie
 */
function wc_mollie() {

	return \WC_Mollie::instance();
}
